// src/installer/extractor.rs
use anyhow::{Result, Context};
use sevenz_rust::decompress;
use std::path::{Path, PathBuf};
use std::sync::Arc;
use tokio::sync::Mutex;
use log::{info, error};
use std::fs;
use std::io::Read;
use crate::app::GameType;

pub struct ExtractionManager {
    progress_callback: Arc<Mutex<Box<dyn Fn(f32, &str) + Send + Sync>>>,
}

impl ExtractionManager {
    pub fn new<F>(progress_callback: F) -> Self 
    where 
        F: Fn(f32, &str) + Send + Sync + 'static 
    {
        Self {
            progress_callback: Arc::new(Mutex::new(Box::new(progress_callback))),
        }
    }
    
    /// Extract a 7z archive with progress reporting
    pub async fn extract_7z(&self, archive_path: &Path, output_dir: &Path) -> Result<()> {
        info!("Starting extraction of 7z archive");
        info!("Archive path: {}", archive_path.display());
        info!("Output directory: {}", output_dir.display());
        
        // Check if archive exists
        if !archive_path.exists() {
            error!("Archive file does not exist: {}", archive_path.display());
            return Err(anyhow::anyhow!("Archive file does not exist"));
        }
        
        // Validate 7z signature
        let mut file = fs::File::open(&archive_path)?;
        let mut signature = [0u8; 6];
        file.read_exact(&mut signature)?;
        
        // 7z files start with bytes [0x37, 0x7A, 0xBC, 0xAF, 0x27, 0x1C]
        if signature != [0x37, 0x7A, 0xBC, 0xAF, 0x27, 0x1C] {
            error!("Invalid 7z file signature: {:?}", signature);
            return Err(anyhow::anyhow!("Invalid 7z file - got wrong signature. The downloaded file might be corrupted or might be an error page"));
        }
        info!("7z signature validation passed");
        
        // Make sure the output directory exists
        if !output_dir.exists() {
            info!("Creating output directory: {}", output_dir.display());
            tokio::fs::create_dir_all(output_dir).await?;
        }
        
        // Get file size for progress calculation
        let file_size = fs::metadata(archive_path)?.len();
        info!("Archive size: {} bytes", file_size);
        
        // We'll handle extraction in a blocking task since sevenz-rust is synchronous
        let archive_path = archive_path.to_path_buf();
        let output_dir = output_dir.to_path_buf();
        let progress_callback = self.progress_callback.clone();
        
        info!("Starting blocking extraction task");
        tokio::task::spawn_blocking(move || -> Result<()> {
            info!("Opening archive file for reading");
            let archive_file = fs::File::open(&archive_path)?;
            
            info!("Starting decompression to: {}", output_dir.display());
            // decompress takes a Read + Seek and a destination directory
            match decompress(&archive_file, &output_dir) {
                Ok(_) => info!("Decompression completed successfully"),
                Err(e) => {
                    error!("Decompression failed: {}", e);
                    return Err(anyhow::anyhow!("Decompression failed: {}", e));
                }
            }

            // List contents of output directory to verify extraction
            if let Ok(entries) = fs::read_dir(&output_dir) {
                info!("Listing extracted contents:");
                for entry in entries {
                    if let Ok(entry) = entry {
                        info!("  {}", entry.path().display());
                    }
                }
            }
        
            // Call progress update with 100% after decompression
            let filename = archive_path.file_name()
                .and_then(|f| f.to_str())
                .unwrap_or("archive");
        
            let progress_callback_clone = progress_callback.clone();
            tokio::runtime::Handle::current().block_on(async move {
                let callback = progress_callback_clone.lock().await;
                callback(1.0, filename); // 100% done
            });
        
            info!("Extraction task completed successfully");
            Ok(())
        }).await??;
        
        info!("7z extraction process completed");
        Ok(())
    }
    
    /// Extract a ZIP archive with progress reporting
    pub async fn extract_zip(&self, archive_path: &Path, output_dir: &Path) -> Result<()> {
        info!("Extracting ZIP archive: {} -> {}", archive_path.display(), output_dir.display());
        
        // Make sure the output directory exists
        if !output_dir.exists() {
            tokio::fs::create_dir_all(output_dir).await?;
        }
        
        // We'll handle extraction in a blocking task since the zip crate is synchronous
        let archive_path = archive_path.to_path_buf();
        let output_dir = output_dir.to_path_buf();
        let progress_callback = self.progress_callback.clone();
        
        tokio::task::spawn_blocking(move || -> Result<()> {
            let file = fs::File::open(&archive_path)?;
            let mut archive = zip::ZipArchive::new(file)?;
            
            let total_entries = archive.len();
            info!("ZIP archive contains {} entries", total_entries);
            
            for i in 0..total_entries {
                let mut file = archive.by_index(i)?;
                let outpath = match file.enclosed_name() {
                    Some(path) => output_dir.join(path),
                    None => continue,
                };
                
                // Update progress
                let progress = (i + 1) as f32 / total_entries as f32;
                let filename = outpath.file_name()
                    .and_then(|f| f.to_str())
                    .unwrap_or("unknown");
                
                if (*file.name()).ends_with('/') {
                    fs::create_dir_all(&outpath)?;
                } else {
                    if let Some(p) = outpath.parent() {
                        if !p.exists() {
                            fs::create_dir_all(p)?;
                        }
                    }
                    let mut outfile = fs::File::create(&outpath)?;
                    std::io::copy(&mut file, &mut outfile)?;
                }
                
                // Fix file permissions on Unix
                #[cfg(unix)]
                {
                    use std::os::unix::fs::PermissionsExt;
                    if let Some(mode) = file.unix_mode() {
                        fs::set_permissions(&outpath, fs::Permissions::from_mode(mode))?;
                    }
                }
                
                // Update progress on the tokio runtime
                let progress_callback_clone = progress_callback.clone();
                tokio::runtime::Handle::current().block_on(async move {
                    let callback = progress_callback_clone.lock().await;
                    callback(progress, filename);
                });
            }
            
            info!("ZIP extraction completed successfully");
            Ok(())
        }).await??;
        
        Ok(())
    }
    
    /// Handles extraction of all downloaded files to their appropriate locations  
    pub async fn install_all_files(
        &self, 
        downloaded_files: &[PathBuf], 
        install_path: &Path, 
        game_type: &GameType, 
        ee_exe_path: Option<&Path>
    ) -> Result<()> {
        info!("Starting installation of all files");
        info!("Install path: {}", install_path.display());
        info!("Game type: {:?}", game_type);
        if let Some(ee_path) = ee_exe_path {
            info!("EE executable path: {}", ee_path.display());
        }

        info!("Available downloaded files:");
        for file in downloaded_files {
            info!("  {}", file.display());
        }

        // Find and extract the main content files
        let content_archive = downloaded_files.iter()
            .find(|p| p.file_name().map_or(false, |f| f.to_str().unwrap_or("").contains("sinfar_all_files")))
            .context("Content archive not found in downloaded files")?;
        
        info!("Found content archive: {}", content_archive.display());
            
        // Extract main content
        info!("Starting extraction of main content");
        self.extract_7z(content_archive, install_path).await?;
        
        // Handle game-specific files
        info!("Processing game-specific files");
        match game_type {
            GameType::Diamond => {
                info!("Installing Diamond edition launcher");
                // For Diamond, find and copy the launcher
                let launcher = downloaded_files.iter()
                    .find(|p| p.file_name().map_or(false, |f| f.to_str().unwrap_or("").contains("sinfarx.exe")))
                    .context("Launcher executable not found in downloaded files")?;
                    
                // Copy launcher to install directory
                let target_path = install_path.join("sinfarx.exe");
                info!("Copying launcher from {} to {}", launcher.display(), target_path.display());
                tokio::fs::copy(launcher, &target_path).await?;
                info!("Launcher copied successfully");
            }
            GameType::EnhancedEdition => {
                info!("Installing Enhanced Edition launcher");
                // For EE, find the launcher zip
                let launcher_zip = downloaded_files.iter()
                    .find(|p| p.file_name().map_or(false, |f| f.to_str().unwrap_or("").contains("sinfarLauncher.zip")))
                    .context("Launcher ZIP not found in downloaded files")?;
                    
                // Extract to the EE executable directory
                if let Some(ee_path) = ee_exe_path {
                    // Create the bin/win32_8181 directory if it doesn't exist (Windows)
                    // or bin/linux_8181 directory (Linux)
                    #[cfg(windows)]
                    let extract_dir = ee_path.join("bin").join("win32_8181");
                    
                    #[cfg(unix)]
                    let extract_dir = ee_path.join("bin").join("linux_8181");
                    
                    info!("Extracting EE launcher to {}", extract_dir.display());
                    // Extract the launcher
                    self.extract_zip(launcher_zip, &extract_dir).await?;
                    info!("EE launcher extracted successfully");
                } else {
                    error!("EE executable path is required but not provided");
                    return Err(anyhow::anyhow!("EE executable path is required for EE installation"));
                }
            }
        }
        
        info!("All files installed successfully");
        Ok(())
    }
}